@@ -23,8 +23,9 @@ from coverage.exceptions import ConfigError
 from coverage.misc import human_sorted_items, isolate_module
 from coverage.plugin import CoveragePlugin
 from coverage.pytracer import PyTracer
+from coverage.sysmon import SysMonitor
 from coverage.types import (
-    TArc, TFileDisposition, TTraceData, TTraceFn, TTracer, TWarnFn,
+    TArc, TFileDisposition, TTraceData, TTraceFn, TracerCore, TWarnFn,
 )
 
 os = isolate_module(os)
@@ -36,14 +37,14 @@ try:
     HAS_CTRACER = True
 except ImportError:
     # Couldn't import the C extension, maybe it isn't built.
-    if os.getenv('COVERAGE_TEST_TRACER') == 'c':        # pragma: part covered
-        # During testing, we use the COVERAGE_TEST_TRACER environment variable
+    if os.getenv("COVERAGE_CORE") == "ctrace":      # pragma: part covered
+        # During testing, we use the COVERAGE_CORE environment variable
         # to indicate that we've fiddled with the environment to test this
         # fallback code.  If we thought we had a C tracer, but couldn't import
         # it, then exit quickly and clearly instead of dribbling confusing
         # errors. I'm using sys.exit here instead of an exception because an
         # exception here causes all sorts of other noise in unittest.
-        sys.stderr.write("*** COVERAGE_TEST_TRACER is 'c' but can't import CTracer!\n")
+        sys.stderr.write("*** COVERAGE_CORE is 'ctrace' but can't import CTracer!\n")
         sys.exit(1)
     HAS_CTRACER = False
 
@@ -129,6 +130,8 @@ class Collector:
         self.concurrency = concurrency
         assert isinstance(self.concurrency, list), f"Expected a list: {self.concurrency!r}"
 
+        self.pid = os.getpid()
+
         self.covdata: CoverageData
         self.threading = None
         self.static_context: Optional[str] = None
@@ -137,24 +140,43 @@ class Collector:
 
         self.concur_id_func = None
 
-        self._trace_class: Type[TTracer]
+        self._trace_class: Type[TracerCore]
         self.file_disposition_class: Type[TFileDisposition]
 
-        use_ctracer = False
-        if HAS_CTRACER and not timid:
-            use_ctracer = True
+        core: Optional[str]
+        if timid:
+            core = "pytrace"
+        else:
+            core = os.getenv("COVERAGE_CORE")
+            if not core:
+                # Once we're comfortable with sysmon as a default:
+                # if env.PYBEHAVIOR.pep669 and self.should_start_context is None:
+                #     core = "sysmon"
+                if HAS_CTRACER:
+                    core = "ctrace"
+                else:
+                    core = "pytrace"
 
-        #if HAS_CTRACER and self._trace_class is CTracer:
-        if use_ctracer:
+        if core == "sysmon":
+            self._trace_class = SysMonitor
+            self.file_disposition_class = FileDisposition
+            self.supports_plugins = False
+            self.packed_arcs = False
+            self.systrace = False
+        elif core == "ctrace":
             self._trace_class = CTracer
             self.file_disposition_class = CFileDisposition
             self.supports_plugins = True
             self.packed_arcs = True
-        else:
+            self.systrace = True
+        elif core == "pytrace":
             self._trace_class = PyTracer
             self.file_disposition_class = FileDisposition
             self.supports_plugins = False
             self.packed_arcs = False
+            self.systrace = True
+        else:
+            raise ConfigError(f"Unknown core value: {core!r}")
 
         # We can handle a few concurrency options here, but only one at a time.
         concurrencies = set(self.concurrency)
@@ -269,11 +291,11 @@ class Collector:
             self.should_trace_cache = {}
 
         # Our active Tracers.
-        self.tracers: List[TTracer] = []
+        self.tracers: List[TracerCore] = []
 
         self._clear_data()
 
-    def _start_tracer(self) -> TTraceFn:
+    def _start_tracer(self) -> TTraceFn | None:
         """Start a new Tracer object, and store it in self.tracers."""
         tracer = self._trace_class()
         tracer.data = self.data
@@ -325,6 +347,17 @@ class Collector:
 
     def start(self) -> None:
         """Start collecting trace information."""
+        # We may be a new collector in a forked process.  The old process'
+        # collectors will be in self._collectors, but they won't be usable.
+        # Find them and discard them.
+        keep_collectors = []
+        for c in self._collectors:
+            if c.pid == self.pid:
+                keep_collectors.append(c)
+            else:
+                c.post_fork()
+        self._collectors[:] = keep_collectors
+
         if self._collectors:
             self._collectors[-1].pause()
 
@@ -344,7 +377,7 @@ class Collector:
 
         # Install our installation tracer in threading, to jump-start other
         # threads.
-        if self.threading:
+        if self.systrace and self.threading:
             self.threading.settrace(self._installation_trace)
 
     def stop(self) -> None:
@@ -360,8 +393,7 @@ class Collector:
 
         self.pause()
 
-        # Remove this Collector from the stack, and resume the one underneath
-        # (if any).
+        # Remove this Collector from the stack, and resume the one underneath (if any).
         self._collectors.pop()
         if self._collectors:
             self._collectors[-1].resume()
@@ -387,6 +419,12 @@ class Collector:
         else:
             self._start_tracer()
 
+    def post_fork(self) -> None:
+        """After a fork, tracers might need to adjust."""
+        for tracer in self.tracers:
+            if hasattr(tracer, "post_fork"):
+                tracer.post_fork()
+
     def _activity(self) -> bool:
         """Has any activity been traced?
 
