@@ -11,6 +11,7 @@ import coverage
 from coverage import env
 from coverage.exceptions import NoDataError
 
+from tests import testenv
 from tests.coveragetest import CoverageTest
 
 
@@ -121,7 +122,8 @@ class BasicCoverageTest(CoverageTest):
             # Nothing here
             d = 6
             """,
-            [1,2,4,6], report="4 0 0 0 100%")
+            [1,2,4,6], report="4 0 0 0 100%"
+        )
 
     def test_indentation_wackiness(self) -> None:
         # Partial final lines are OK.
@@ -130,7 +132,8 @@ class BasicCoverageTest(CoverageTest):
             if not sys.path:
                 a = 1
                 """,    # indented last line
-            [1,2,3], "3")
+            [1,2,3], "3"
+        )
 
     def test_multiline_initializer(self) -> None:
         self.check_coverage("""\
@@ -142,7 +145,8 @@ class BasicCoverageTest(CoverageTest):
 
             e = { 'foo': 1, 'bar': 2 }
             """,
-            [1,7], "")
+            [1,7], ""
+        )
 
     def test_list_comprehension(self) -> None:
         self.check_coverage("""\
@@ -152,7 +156,8 @@ class BasicCoverageTest(CoverageTest):
                 ]
             assert l == [12, 14, 16, 18]
             """,
-            [1,5], "")
+            [1,5], ""
+        )
 
 
 class SimpleStatementTest(CoverageTest):
@@ -166,26 +171,30 @@ class SimpleStatementTest(CoverageTest):
             12
             23
             """,
-            ([1,2],[2]), "")
+            ([1,2],[2]), ""
+        )
         self.check_coverage("""\
             12
             23
             a = 3
             """,
-            ([1,2,3],[3]), "")
+            ([1,2,3],[3]), ""
+        )
         self.check_coverage("""\
             1 + 2
             1 + \\
                 2
             """,
-            ([1,2], [2]), "")
+            ([1,2], [2]), ""
+        )
         self.check_coverage("""\
             1 + 2
             1 + \\
                 2
             a = 4
             """,
-            ([1,2,4], [4]), "")
+            ([1,2,4], [4]), ""
+        )
 
     def test_assert(self) -> None:
         self.check_coverage("""\
@@ -197,7 +206,8 @@ class SimpleStatementTest(CoverageTest):
                 2), \\
                 'something is amiss'
             """,
-            [1,2,4,5], "")
+            [1,2,4,5], ""
+        )
 
     def test_assignment(self) -> None:
         # Simple variable assignment
@@ -208,7 +218,8 @@ class SimpleStatementTest(CoverageTest):
             c = \\
                 1
             """,
-            [1,2,4], "")
+            [1,2,4], ""
+        )
 
     def test_assign_tuple(self) -> None:
         self.check_coverage("""\
@@ -216,7 +227,8 @@ class SimpleStatementTest(CoverageTest):
             a,b,c = 7,8,9
             assert a == 7 and b == 8 and c == 9
             """,
-            [1,2,3], "")
+            [1,2,3], ""
+        )
 
     def test_more_assignments(self) -> None:
         self.check_coverage("""\
@@ -231,7 +243,8 @@ class SimpleStatementTest(CoverageTest):
             ] = \\
                 9
             """,
-            [1, 2, 3], "")
+            [1, 2, 3], ""
+        )
 
     def test_attribute_assignment(self) -> None:
         # Attribute assignment
@@ -244,7 +257,8 @@ class SimpleStatementTest(CoverageTest):
             o.foo = \\
                 1
             """,
-            [1,2,3,4,6], "")
+            [1,2,3,4,6], ""
+        )
 
     def test_list_of_attribute_assignment(self) -> None:
         self.check_coverage("""\
@@ -258,7 +272,8 @@ class SimpleStatementTest(CoverageTest):
                 1, \\
                 2
             """,
-            [1,2,3,4,7], "")
+            [1,2,3,4,7], ""
+        )
 
     def test_augmented_assignment(self) -> None:
         self.check_coverage("""\
@@ -269,7 +284,8 @@ class SimpleStatementTest(CoverageTest):
             a += \\
                 1
             """,
-            [1,2,3,5], "")
+            [1,2,3,5], ""
+        )
 
     def test_triple_string_stuff(self) -> None:
         self.check_coverage("""\
@@ -291,7 +307,8 @@ class SimpleStatementTest(CoverageTest):
                 lines.
                 ''')
             """,
-            [1,5,11], "")
+            [1,5,11], ""
+        )
 
     def test_pass(self) -> None:
         # pass is tricky: if it's the only statement in a block, then it is
@@ -300,27 +317,31 @@ class SimpleStatementTest(CoverageTest):
             if 1==1:
                 pass
             """,
-            [1,2], "")
+            [1,2], ""
+        )
         self.check_coverage("""\
             def foo():
                 pass
             foo()
             """,
-            [1,2,3], "")
+            [1,2,3], ""
+        )
         self.check_coverage("""\
             def foo():
                 "doc"
                 pass
             foo()
             """,
-            ([1,3,4], [1,4]), "")
+            ([1,3,4], [1,4]), ""
+        )
         self.check_coverage("""\
             class Foo:
                 def foo(self):
                     pass
             Foo().foo()
             """,
-            [1,2,3,4], "")
+            [1,2,3,4], ""
+        )
         self.check_coverage("""\
             class Foo:
                 def foo(self):
@@ -328,7 +349,8 @@ class SimpleStatementTest(CoverageTest):
                     pass
             Foo().foo()
             """,
-            ([1,2,4,5], [1,2,5]), "")
+            ([1,2,4,5], [1,2,5]), ""
+        )
 
     def test_del(self) -> None:
         self.check_coverage("""\
@@ -342,7 +364,8 @@ class SimpleStatementTest(CoverageTest):
                 d['e']
             assert(len(d.keys()) == 0)
             """,
-            [1,2,3,6,9], "")
+            [1,2,3,6,9], ""
+        )
 
     def test_raise(self) -> None:
         self.check_coverage("""\
@@ -353,7 +376,8 @@ class SimpleStatementTest(CoverageTest):
             except:
                 pass
             """,
-            [1,2,5,6], "")
+            [1,2,5,6], ""
+        )
 
     def test_raise_followed_by_statement(self) -> None:
         if env.PYBEHAVIOR.omit_after_jump:
@@ -369,7 +393,8 @@ class SimpleStatementTest(CoverageTest):
             except:
                 pass
             """,
-            lines=lines, missing=missing)
+            lines=lines, missing=missing
+        )
 
     def test_return(self) -> None:
         self.check_coverage("""\
@@ -380,7 +405,8 @@ class SimpleStatementTest(CoverageTest):
             x = fn()
             assert(x == 1)
             """,
-            [1,2,3,5,6], "")
+            [1,2,3,5,6], ""
+        )
         self.check_coverage("""\
             def fn():
                 a = 1
@@ -391,7 +417,8 @@ class SimpleStatementTest(CoverageTest):
             x = fn()
             assert(x == 2)
             """,
-            [1,2,3,7,8], "")
+            [1,2,3,7,8], ""
+        )
         self.check_coverage("""\
             def fn():
                 a = 1
@@ -402,7 +429,8 @@ class SimpleStatementTest(CoverageTest):
             x,y,z = fn()
             assert x == 1 and y == 2 and z == 3
             """,
-            [1,2,3,7,8], "")
+            [1,2,3,7,8], ""
+        )
 
     def test_return_followed_by_statement(self) -> None:
         if env.PYBEHAVIOR.omit_after_return:
@@ -435,7 +463,8 @@ class SimpleStatementTest(CoverageTest):
             a,b,c = gen()
             assert a == 1 and b == 9 and c == (1,2)
             """,
-            [1,2,3,6,8,9], "")
+            [1,2,3,6,8,9], ""
+        )
 
     def test_break(self) -> None:
         if env.PYBEHAVIOR.omit_after_jump:
@@ -452,7 +481,8 @@ class SimpleStatementTest(CoverageTest):
                 a = 4
             assert a == 2
             """,
-            lines=lines, missing=missing)
+            lines=lines, missing=missing
+        )
 
     def test_continue(self) -> None:
         if env.PYBEHAVIOR.omit_after_jump:
@@ -469,7 +499,8 @@ class SimpleStatementTest(CoverageTest):
                 a = 4
             assert a == 11
             """,
-            lines=lines, missing=missing)
+            lines=lines, missing=missing
+        )
 
     def test_strange_unexecuted_continue(self) -> None:
         # Peephole optimization of jumps to jumps can mean that some statements
@@ -508,14 +539,16 @@ class SimpleStatementTest(CoverageTest):
             from sys import path
             a = 1
             """,
-            [1,2,3], "")
+            [1,2,3], ""
+        )
         self.check_coverage("""\
             import string
             if 1 == 2:
                 from sys import path
             a = 1
             """,
-            [1,2,3,4], "3")
+            [1,2,3,4], "3"
+        )
         self.check_coverage("""\
             import string, \\
                 os, \\
@@ -524,30 +557,35 @@ class SimpleStatementTest(CoverageTest):
                 stdout
             a = 1
             """,
-            [1,4,6], "")
+            [1,4,6], ""
+        )
         self.check_coverage("""\
             import sys, sys as s
             assert s.path == sys.path
             """,
-            [1,2], "")
+            [1,2], ""
+        )
         self.check_coverage("""\
             import sys, \\
                 sys as s
             assert s.path == sys.path
             """,
-            [1,3], "")
+            [1,3], ""
+        )
         self.check_coverage("""\
             from sys import path, \\
                 path as p
             assert p == path
             """,
-            [1,3], "")
+            [1,3], ""
+        )
         self.check_coverage("""\
             from sys import \\
                 *
             assert len(path) > 0
             """,
-            [1,3], "")
+            [1,3], ""
+        )
 
     def test_global(self) -> None:
         self.check_coverage("""\
@@ -560,7 +598,8 @@ class SimpleStatementTest(CoverageTest):
             fn()
             assert g == 2 and h == 2 and i == 2
             """,
-            [1,2,6,7,8], "")
+            [1,2,6,7,8], ""
+        )
         self.check_coverage("""\
             g = h = i = 1
             def fn():
@@ -568,7 +607,8 @@ class SimpleStatementTest(CoverageTest):
             fn()
             assert g == 2 and h == 1 and i == 1
             """,
-            [1,2,3,4,5], "")
+            [1,2,3,4,5], ""
+        )
 
     def test_exec(self) -> None:
         self.check_coverage("""\
@@ -579,7 +619,8 @@ class SimpleStatementTest(CoverageTest):
                 "2")
             assert a == 2 and b == 2 and c == 2
             """,
-            [1,2,3,6], "")
+            [1,2,3,6], ""
+        )
         self.check_coverage("""\
             vars = {'a': 1, 'b': 1, 'c': 1}
             exec("a = 2", vars)
@@ -588,7 +629,8 @@ class SimpleStatementTest(CoverageTest):
                 "2", vars)
             assert vars['a'] == 2 and vars['b'] == 2 and vars['c'] == 2
             """,
-            [1,2,3,6], "")
+            [1,2,3,6], ""
+        )
         self.check_coverage("""\
             globs = {}
             locs = {'a': 1, 'b': 1, 'c': 1}
@@ -598,7 +640,8 @@ class SimpleStatementTest(CoverageTest):
                 "2", globs, locs)
             assert locs['a'] == 2 and locs['b'] == 2 and locs['c'] == 2
             """,
-            [1,2,3,4,7], "")
+            [1,2,3,4,7], ""
+        )
 
     def test_extra_doc_string(self) -> None:
         self.check_coverage("""\
@@ -662,7 +705,8 @@ class CompoundStatementTest(CoverageTest):
 
             assert (a,b,c,d,e) == (1,2,3,4,5)
             """,
-            [1,2,3,5], "")
+            [1,2,3,5], ""
+        )
 
     def test_if(self) -> None:
         self.check_coverage("""\
@@ -675,7 +719,8 @@ class CompoundStatementTest(CoverageTest):
                 x = 7
             assert x == 7
             """,
-            [1,2,3,4,5,7,8], "")
+            [1,2,3,4,5,7,8], ""
+        )
         self.check_coverage("""\
             a = 1
             if a == 1:
@@ -684,7 +729,8 @@ class CompoundStatementTest(CoverageTest):
                 y = 5
             assert x == 3
             """,
-            [1,2,3,5,6], "5")
+            [1,2,3,5,6], "5"
+        )
         self.check_coverage("""\
             a = 1
             if a != 1:
@@ -693,7 +739,8 @@ class CompoundStatementTest(CoverageTest):
                 y = 5
             assert y == 5
             """,
-            [1,2,3,5,6], "3")
+            [1,2,3,5,6], "3"
+        )
         self.check_coverage("""\
             a = 1; b = 2
             if a == 1:
@@ -705,7 +752,8 @@ class CompoundStatementTest(CoverageTest):
                 z = 8
             assert x == 4
             """,
-            [1,2,3,4,6,8,9], "6-8")
+            [1,2,3,4,6,8,9], "6-8"
+        )
 
     def test_elif(self) -> None:
         self.check_coverage("""\
@@ -783,7 +831,8 @@ class CompoundStatementTest(CoverageTest):
                 else:
                     x = 13
             """,
-            [1,2,3,4,5,6,7,8,9,10,11,13], "2-13")
+            [1,2,3,4,5,6,7,8,9,10,11,13], "2-13"
+        )
 
     def test_split_if(self) -> None:
         self.check_coverage("""\
@@ -798,7 +847,8 @@ class CompoundStatementTest(CoverageTest):
                 z = 7
             assert x == 3
             """,
-            [1,2,4,5,7,9,10], "5-9")
+            [1,2,4,5,7,9,10], "5-9"
+        )
         self.check_coverage("""\
             a = 1; b = 2; c = 3;
             if \\
@@ -811,7 +861,8 @@ class CompoundStatementTest(CoverageTest):
                 z = 7
             assert y == 5
             """,
-            [1,2,4,5,7,9,10], "4, 9")
+            [1,2,4,5,7,9,10], "4, 9"
+        )
         self.check_coverage("""\
             a = 1; b = 2; c = 3;
             if \\
@@ -824,7 +875,8 @@ class CompoundStatementTest(CoverageTest):
                 z = 7
             assert z == 7
             """,
-            [1,2,4,5,7,9,10], "4, 7")
+            [1,2,4,5,7,9,10], "4, 7"
+        )
 
     def test_pathological_split_if(self) -> None:
         self.check_coverage("""\
@@ -841,7 +893,8 @@ class CompoundStatementTest(CoverageTest):
                 z = 7
             assert x == 3
             """,
-            [1,2,5,6,9,11,12], "6-11")
+            [1,2,5,6,9,11,12], "6-11"
+        )
         self.check_coverage("""\
             a = 1; b = 2; c = 3;
             if (
@@ -856,7 +909,8 @@ class CompoundStatementTest(CoverageTest):
                 z = 7
             assert y == 5
             """,
-            [1,2,5,6,9,11,12], "5, 11")
+            [1,2,5,6,9,11,12], "5, 11"
+        )
         self.check_coverage("""\
             a = 1; b = 2; c = 3;
             if (
@@ -871,7 +925,8 @@ class CompoundStatementTest(CoverageTest):
                 z = 7
             assert z == 7
             """,
-            [1,2,5,6,9,11,12], "5, 9")
+            [1,2,5,6,9,11,12], "5, 9"
+        )
 
     def test_absurd_split_if(self) -> None:
         self.check_coverage("""\
@@ -886,7 +941,8 @@ class CompoundStatementTest(CoverageTest):
                 z = 7
             assert x == 3
             """,
-            [1,2,4,5,7,9,10], "5-9")
+            [1,2,4,5,7,9,10], "5-9"
+        )
         self.check_coverage("""\
             a = 1; b = 2; c = 3;
             if a != 1 \\
@@ -899,7 +955,8 @@ class CompoundStatementTest(CoverageTest):
                 z = 7
             assert y == 5
             """,
-            [1,2,4,5,7,9,10], "4, 9")
+            [1,2,4,5,7,9,10], "4, 9"
+        )
         self.check_coverage("""\
             a = 1; b = 2; c = 3;
             if a != 1 \\
@@ -912,7 +969,8 @@ class CompoundStatementTest(CoverageTest):
                 z = 7
             assert z == 7
             """,
-            [1,2,4,5,7,9,10], "4, 7")
+            [1,2,4,5,7,9,10], "4, 7"
+        )
 
     def test_constant_if(self) -> None:
         if env.PYBEHAVIOR.keep_constant_test:
@@ -936,7 +994,8 @@ class CompoundStatementTest(CoverageTest):
                 a -= 1
             assert a == 0 and b == 3
             """,
-            [1,2,3,4,5], "")
+            [1,2,3,4,5], ""
+        )
         self.check_coverage("""\
             a = 3; b = 0
             while a:
@@ -944,7 +1003,8 @@ class CompoundStatementTest(CoverageTest):
                 break
             assert a == 3 and b == 1
             """,
-            [1,2,3,4,5], "")
+            [1,2,3,4,5], ""
+        )
 
     def test_while_else(self) -> None:
         # Take the else branch.
@@ -957,7 +1017,8 @@ class CompoundStatementTest(CoverageTest):
                 b = 99
             assert a == 0 and b == 99
             """,
-            [1,2,3,4,6,7], "")
+            [1,2,3,4,6,7], ""
+        )
         # Don't take the else branch.
         self.check_coverage("""\
             a = 3; b = 0
@@ -969,7 +1030,8 @@ class CompoundStatementTest(CoverageTest):
                 b = 99
             assert a == 2 and b == 1
             """,
-            [1,2,3,4,5,7,8], "7")
+            [1,2,3,4,5,7,8], "7"
+        )
 
     def test_split_while(self) -> None:
         self.check_coverage("""\
@@ -980,7 +1042,8 @@ class CompoundStatementTest(CoverageTest):
                 a -= 1
             assert a == 0 and b == 3
             """,
-            [1,2,4,5,6], "")
+            [1,2,4,5,6], ""
+        )
         self.check_coverage("""\
             a = 3; b = 0
             while (
@@ -990,7 +1053,8 @@ class CompoundStatementTest(CoverageTest):
                 a -= 1
             assert a == 0 and b == 3
             """,
-            [1,2,5,6,7], "")
+            [1,2,5,6,7], ""
+        )
 
     def test_for(self) -> None:
         self.check_coverage("""\
@@ -999,7 +1063,8 @@ class CompoundStatementTest(CoverageTest):
                 a += i
             assert a == 15
             """,
-            [1,2,3,4], "")
+            [1,2,3,4], ""
+        )
         self.check_coverage("""\
             a = 0
             for i in [1,
@@ -1008,7 +1073,8 @@ class CompoundStatementTest(CoverageTest):
                 a += i
             assert a == 15
             """,
-            [1,2,5,6], "")
+            [1,2,5,6], ""
+        )
         self.check_coverage("""\
             a = 0
             for i in [1,2,3,4,5]:
@@ -1016,7 +1082,8 @@ class CompoundStatementTest(CoverageTest):
                 break
             assert a == 1
             """,
-            [1,2,3,4,5], "")
+            [1,2,3,4,5], ""
+        )
 
     def test_for_else(self) -> None:
         self.check_coverage("""\
@@ -1027,7 +1094,8 @@ class CompoundStatementTest(CoverageTest):
                 a = 99
             assert a == 99
             """,
-            [1,2,3,5,6], "")
+            [1,2,3,5,6], ""
+        )
         self.check_coverage("""\
             a = 0
             for i in range(5):
@@ -1037,7 +1105,8 @@ class CompoundStatementTest(CoverageTest):
                 a = 123
             assert a == 1
             """,
-            [1,2,3,4,6,7], "6")
+            [1,2,3,4,6,7], "6"
+        )
 
     def test_split_for(self) -> None:
         self.check_coverage("""\
@@ -1047,7 +1116,8 @@ class CompoundStatementTest(CoverageTest):
                 a += i
             assert a == 15
             """,
-            [1,2,4,5], "")
+            [1,2,4,5], ""
+        )
         self.check_coverage("""\
             a = 0
             for \\
@@ -1057,7 +1127,8 @@ class CompoundStatementTest(CoverageTest):
                 a += i
             assert a == 15
             """,
-            [1,2,6,7], "")
+            [1,2,6,7], ""
+        )
 
     def test_try_except(self) -> None:
         self.check_coverage("""\
@@ -1068,7 +1139,8 @@ class CompoundStatementTest(CoverageTest):
                 a = 99
             assert a == 1
             """,
-            [1,2,3,4,5,6], "4-5")
+            [1,2,3,4,5,6], "4-5"
+        )
         self.check_coverage("""\
             a = 0
             try:
@@ -1078,7 +1150,8 @@ class CompoundStatementTest(CoverageTest):
                 a = 99
             assert a == 99
             """,
-            [1,2,3,4,5,6,7], "")
+            [1,2,3,4,5,6,7], ""
+        )
         self.check_coverage("""\
             a = 0
             try:
@@ -1090,7 +1163,8 @@ class CompoundStatementTest(CoverageTest):
                 a = 123
             assert a == 123
             """,
-            [1,2,3,4,5,6,7,8,9], "6")
+            [1,2,3,4,5,6,7,8,9], "6"
+        )
         self.check_coverage("""\
             a = 0
             try:
@@ -1104,7 +1178,8 @@ class CompoundStatementTest(CoverageTest):
                 a = 123
             assert a == 17
             """,
-            [1,2,3,4,5,6,7,8,9,10,11], "6, 9-10")
+            [1,2,3,4,5,6,7,8,9,10,11], "6, 9-10"
+        )
         self.check_coverage("""\
             a = 0
             try:
@@ -1121,7 +1196,7 @@ class CompoundStatementTest(CoverageTest):
         )
 
     def test_try_except_stranded_else(self) -> None:
-        if env.PYBEHAVIOR.omit_after_jump:
+        if env.PYBEHAVIOR.optimize_unreachable_try_else:
             # The else can't be reached because the try ends with a raise.
             lines = [1,2,3,4,5,6,9]
             missing = ""
@@ -1158,7 +1233,8 @@ class CompoundStatementTest(CoverageTest):
                 a = 99
             assert a == 99
             """,
-            [1,2,3,5,6], "")
+            [1,2,3,5,6], ""
+        )
         self.check_coverage("""\
             a = 0; b = 0
             try:
@@ -1171,7 +1247,8 @@ class CompoundStatementTest(CoverageTest):
                 a = 99
             assert a == 99 and b == 123
             """,
-            [1,2,3,4,5,7,8,9,10], "")
+            [1,2,3,4,5,7,8,9,10], ""
+        )
 
     def test_function_def(self) -> None:
         self.check_coverage("""\
@@ -1184,7 +1261,8 @@ class CompoundStatementTest(CoverageTest):
             a = foo()
             assert a == 1
             """,
-            [1,2,5,7,8], "")
+            [1,2,5,7,8], ""
+        )
         self.check_coverage("""\
             def foo(
                 a,
@@ -1197,7 +1275,8 @@ class CompoundStatementTest(CoverageTest):
             x = foo(17, 23)
             assert x == 40
             """,
-            [1,7,9,10], "")
+            [1,7,9,10], ""
+        )
         self.check_coverage("""\
             def foo(
                 a = (lambda x: x*2)(10),
@@ -1213,7 +1292,8 @@ class CompoundStatementTest(CoverageTest):
             x = foo()
             assert x == 22
             """,
-            [1,10,12,13], "")
+            [1,10,12,13], ""
+        )
 
     def test_class_def(self) -> None:
         arcz="-22 2D DE E-2  23 36 6A A-2  -68 8-6   -AB B-A"
@@ -1255,15 +1335,6 @@ class ExcludeTest(CoverageTest):
             [1,3,5,7]
         )
 
-    def test_simple(self) -> None:
-        self.check_coverage("""\
-            a = 1; b = 2
-
-            if len([]):
-                a = 4   # -cc
-            """,
-            [1,3], "", excludes=['-cc'])
-
     def test_two_excludes(self) -> None:
         self.check_coverage("""\
             a = 1; b = 2
@@ -1274,69 +1345,8 @@ class ExcludeTest(CoverageTest):
                 c = 6   # -xx
             assert a == 1 and b == 2
             """,
-            [1,3,5,7], "5", excludes=['-cc', '-xx'])
-
-    def test_excluding_if_suite(self) -> None:
-        self.check_coverage("""\
-            a = 1; b = 2
-
-            if len([]):     # not-here
-                a = 4
-                b = 5
-                c = 6
-            assert a == 1 and b == 2
-            """,
-            [1,7], "", excludes=['not-here'])
-
-    def test_excluding_if_but_not_else_suite(self) -> None:
-        self.check_coverage("""\
-            a = 1; b = 2
-
-            if len([]):     # not-here
-                a = 4
-                b = 5
-                c = 6
-            else:
-                a = 8
-                b = 9
-            assert a == 8 and b == 9
-            """,
-            [1,8,9,10], "", excludes=['not-here'])
-
-    def test_excluding_else_suite(self) -> None:
-        self.check_coverage("""\
-            a = 1; b = 2
-
-            if 1==1:
-                a = 4
-                b = 5
-                c = 6
-            else:          #pragma: NO COVER
-                a = 8
-                b = 9
-            assert a == 4 and b == 5 and c == 6
-            """,
-            [1,3,4,5,6,10], "", excludes=['#pragma: NO COVER'])
-        self.check_coverage("""\
-            a = 1; b = 2
-
-            if 1==1:
-                a = 4
-                b = 5
-                c = 6
-
-            # Lots of comments to confuse the else handler.
-            # more.
-
-            else:          #pragma: NO COVER
-
-            # Comments here too.
-
-                a = 8
-                b = 9
-            assert a == 4 and b == 5 and c == 6
-            """,
-            [1,3,4,5,6,17], "", excludes=['#pragma: NO COVER'])
+            [1,3,5,7], "5", excludes=['-cc', '-xx']
+        )
 
     def test_excluding_elif_suites(self) -> None:
         self.check_coverage("""\
@@ -1354,134 +1364,10 @@ class ExcludeTest(CoverageTest):
                 b = 12
             assert a == 4 and b == 5 and c == 6
             """,
-            [1,3,4,5,6,11,12,13], "11-12", excludes=['#pragma: NO COVER'])
-
-    def test_excluding_oneline_if(self) -> None:
-        self.check_coverage("""\
-            def foo():
-                a = 2
-                if len([]): x = 3       # no cover
-                b = 4
-
-            foo()
-            """,
-            [1,2,4,6], "", excludes=["no cover"])
-
-    def test_excluding_a_colon_not_a_suite(self) -> None:
-        self.check_coverage("""\
-            def foo():
-                l = list(range(10))
-                a = l[:3]   # no cover
-                b = 4
-
-            foo()
-            """,
-            [1,2,4,6], "", excludes=["no cover"])
-
-    def test_excluding_for_suite(self) -> None:
-        self.check_coverage("""\
-            a = 0
-            for i in [1,2,3,4,5]:     #pragma: NO COVER
-                a += i
-            assert a == 15
-            """,
-            [1,4], "", excludes=['#pragma: NO COVER'])
-        self.check_coverage("""\
-            a = 0
-            for i in [1,
-                2,3,4,
-                5]:                #pragma: NO COVER
-                a += i
-            assert a == 15
-            """,
-            [1,6], "", excludes=['#pragma: NO COVER'])
-        self.check_coverage("""\
-            a = 0
-            for i in [1,2,3,4,5
-                ]:                        #pragma: NO COVER
-                a += i
-                break
-                a = 99
-            assert a == 1
-            """,
-            [1,7], "", excludes=['#pragma: NO COVER'])
-
-    def test_excluding_for_else(self) -> None:
-        self.check_coverage("""\
-            a = 0
-            for i in range(5):
-                a += i+1
-                break
-            else:               #pragma: NO COVER
-                a = 123
-            assert a == 1
-            """,
-            [1,2,3,4,7], "", excludes=['#pragma: NO COVER'])
-
-    def test_excluding_while(self) -> None:
-        self.check_coverage("""\
-            a = 3; b = 0
-            while a*b:           #pragma: NO COVER
-                b += 1
-                break
-            assert a == 3 and b == 0
-            """,
-            [1,5], "", excludes=['#pragma: NO COVER'])
-        self.check_coverage("""\
-            a = 3; b = 0
-            while (
-                a*b
-                ):           #pragma: NO COVER
-                b += 1
-                break
-            assert a == 3 and b == 0
-            """,
-            [1,7], "", excludes=['#pragma: NO COVER'])
-
-    def test_excluding_while_else(self) -> None:
-        self.check_coverage("""\
-            a = 3; b = 0
-            while a:
-                b += 1
-                break
-            else:           #pragma: NO COVER
-                b = 123
-            assert a == 3 and b == 1
-            """,
-            [1,2,3,4,7], "", excludes=['#pragma: NO COVER'])
+            [1,3,4,5,6,11,12,13], "11-12", excludes=['#pragma: NO COVER']
+        )
 
     def test_excluding_try_except(self) -> None:
-        self.check_coverage("""\
-            a = 0
-            try:
-                a = 1
-            except:           #pragma: NO COVER
-                a = 99
-            assert a == 1
-            """,
-            [1,2,3,6], "", excludes=['#pragma: NO COVER'])
-        self.check_coverage("""\
-            a = 0
-            try:
-                a = 1
-                raise Exception("foo")
-            except:
-                a = 99
-            assert a == 99
-            """,
-            [1,2,3,4,5,6,7], "", excludes=['#pragma: NO COVER'])
-        self.check_coverage("""\
-            a = 0
-            try:
-                a = 1
-                raise Exception("foo")
-            except ImportError:    #pragma: NO COVER
-                a = 99
-            except:
-                a = 123
-            assert a == 123
-            """,
-            [1,2,3,4,7,8,9], "", excludes=['#pragma: NO COVER'])
         self.check_coverage("""\
             a = 0
             try:
@@ -1498,7 +1384,7 @@ class ExcludeTest(CoverageTest):
         )
 
     def test_excluding_try_except_stranded_else(self) -> None:
-        if env.PYBEHAVIOR.omit_after_jump:
+        if env.PYBEHAVIOR.optimize_unreachable_try_else:
             # The else can't be reached because the try ends with a raise.
             arcz = ".1 12 23 34 45 56 69 9."
             arcz_missing = ""
@@ -1521,86 +1407,7 @@ class ExcludeTest(CoverageTest):
             arcz_missing=arcz_missing,
         )
 
-    def test_excluding_if_pass(self) -> None:
-        # From a comment on the coverage.py page by Michael McNeil Forbes:
-        self.check_coverage("""\
-            def f():
-                if False:    # pragma: no cover
-                    pass     # This line still reported as missing
-                if False:    # pragma: no cover
-                    x = 1    # Now it is skipped.
-
-            f()
-            """,
-            [1,7], "", excludes=["no cover"])
-
-    def test_excluding_function(self) -> None:
-        self.check_coverage("""\
-            def fn(foo):      #pragma: NO COVER
-                a = 1
-                b = 2
-                c = 3
-
-            x = 1
-            assert x == 1
-            """,
-            [6,7], "", excludes=['#pragma: NO COVER'])
-
-    def test_excluding_method(self) -> None:
-        self.check_coverage("""\
-            class Fooey:
-                def __init__(self):
-                    self.a = 1
-
-                def foo(self):     #pragma: NO COVER
-                    return self.a
-
-            x = Fooey()
-            assert x.a == 1
-            """,
-            [1,2,3,8,9], "", excludes=['#pragma: NO COVER'])
-
-    def test_excluding_class(self) -> None:
-        self.check_coverage("""\
-            class Fooey:            #pragma: NO COVER
-                def __init__(self):
-                    self.a = 1
-
-                def foo(self):
-                    return self.a
-
-            x = 1
-            assert x == 1
-            """,
-            [8,9], "", excludes=['#pragma: NO COVER'])
-
-    def test_excludes_non_ascii(self) -> None:
-        self.check_coverage("""\
-            # coding: utf-8
-            a = 1; b = 2
-
-            if len([]):
-                a = 5   # ✘cover
-            """,
-            [2, 4], "", excludes=['✘cover']
-        )
-
-    def test_formfeed(self) -> None:
-        # https://github.com/nedbat/coveragepy/issues/461
-        self.check_coverage("""\
-            x = 1
-            assert len([]) == 0, (
-                "This won't happen %s" % ("hello",)
-            )
-            \f
-            x = 6
-            assert len([]) == 0, (
-                "This won't happen %s" % ("hello",)
-            )
-            """,
-            [1, 6], "", excludes=['assert'],
-        )
-
+    @pytest.mark.xfail(testenv.SYS_MON, reason="TODO: fix this for sys.monitoring")
     def test_excluded_comprehension_branches(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/1271
         self.check_coverage("""\
@@ -1620,9 +1427,7 @@ class Py24Test(CoverageTest):
     """Tests of new syntax in Python 2.4."""
 
     def test_function_decorators(self) -> None:
-        lines = [1, 2, 3, 4, 6, 8, 10, 12]
-        if env.PYBEHAVIOR.trace_decorated_def:
-            lines = sorted(lines + [9])
+        lines = [1, 2, 3, 4, 6, 8, 9, 10, 12]
         self.check_coverage("""\
             def require_int(func):
                 def wrapper(arg):
@@ -1637,12 +1442,11 @@ class Py24Test(CoverageTest):
 
             assert p1(10) == 20
             """,
-            lines, "")
+            lines, ""
+        )
 
     def test_function_decorators_with_args(self) -> None:
-        lines = [1, 2, 3, 4, 5, 6, 8, 10, 12]
-        if env.PYBEHAVIOR.trace_decorated_def:
-            lines = sorted(lines + [9])
+        lines = [1, 2, 3, 4, 5, 6, 8, 9, 10, 12]
         self.check_coverage("""\
             def boost_by(extra):
                 def decorator(func):
@@ -1657,12 +1461,11 @@ class Py24Test(CoverageTest):
 
             assert boosted(10) == 200
             """,
-            lines, "")
+            lines, ""
+        )
 
     def test_double_function_decorators(self) -> None:
-        lines = [1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, 17, 19, 21, 22, 24, 26]
-        if env.PYBEHAVIOR.trace_decorated_def:
-            lines = sorted(lines + [16, 23])
+        lines = [1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, 16, 17, 19, 21, 22, 23, 24, 26]
         self.check_coverage("""\
             def require_int(func):
                 def wrapper(arg):
@@ -1691,7 +1494,8 @@ class Py24Test(CoverageTest):
 
             assert boosted2(10) == 200
             """,
-            lines, "")
+            lines, ""
+        )
 
 
 class Py25Test(CoverageTest):
@@ -1718,7 +1522,8 @@ class Py25Test(CoverageTest):
             except:
                 desc = "caught"
             """,
-            [1,2,3,5,6,8,9,10,11,13,14,15,16,17,18], "")
+            [1,2,3,5,6,8,9,10,11,13,14,15,16,17,18], ""
+        )
 
     def test_try_except_finally(self) -> None:
         self.check_coverage("""\
@@ -1801,7 +1606,7 @@ class Py25Test(CoverageTest):
         )
 
     def test_try_except_finally_stranded_else(self) -> None:
-        if env.PYBEHAVIOR.omit_after_jump:
+        if env.PYBEHAVIOR.optimize_unreachable_try_else:
             # The else can't be reached because the try ends with a raise.
             lines = [1,2,3,4,5,6,10,11]
             missing = ""
