@@ -18,9 +18,9 @@ from coverage.data import sorted_lines
 from coverage.files import abs_file
 from coverage.misc import import_local_file
 
+from tests import osinfo, testenv
 from tests.coveragetest import CoverageTest
 from tests.helpers import swallow_warnings
-from tests import osinfo
 
 
 class ThreadingTest(CoverageTest):
@@ -162,7 +162,7 @@ class MemoryLeakTest(CoverageTest):
 
     """
     @flaky      # type: ignore[misc]
-    @pytest.mark.skipif(not env.C_TRACER, reason="Only the C tracer has refcounting issues")
+    @pytest.mark.skipif(not testenv.C_TRACER, reason="Only the C tracer has refcounting issues")
     def test_for_leaks(self) -> None:
         # Our original bad memory leak only happened on line numbers > 255, so
         # make a code object with more lines than that.  Ugly string mumbo
@@ -209,7 +209,7 @@ class MemoryLeakTest(CoverageTest):
 class MemoryFumblingTest(CoverageTest):
     """Test that we properly manage the None refcount."""
 
-    @pytest.mark.skipif(not env.C_TRACER, reason="Only the C tracer has refcounting issues")
+    @pytest.mark.skipif(not testenv.C_TRACER, reason="Only the C tracer has refcounting issues")
     def test_dropping_none(self) -> None:                     # pragma: not covered
         # TODO: Mark this so it will only be run sometimes.
         pytest.skip("This is too expensive for now (30s)")
@@ -474,21 +474,25 @@ class GettraceTest(CoverageTest):
 
     def test_setting_new_trace_function(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/436
+        if testenv.SETTRACE_CORE:
+            missing = "5-7, 13-14"
+        else:
+            missing = "5-7"
         self.check_coverage('''\
             import os.path
             import sys
 
             def tracer(frame, event, arg):
-                filename = os.path.basename(frame.f_code.co_filename)
-                print(f"{event}: {filename} @ {frame.f_lineno}")
-                return tracer
+                filename = os.path.basename(frame.f_code.co_filename)   # 5
+                print(f"{event}: {filename} @ {frame.f_lineno}")        # 6
+                return tracer                                           # 7
 
             def begin():
                 sys.settrace(tracer)
 
             def collect():
-                t = sys.gettrace()
-                assert t is tracer, t
+                t = sys.gettrace()              # 13
+                assert t is tracer, t           # 14
 
             def test_unsets_trace() -> None:
                 begin()
@@ -501,7 +505,7 @@ class GettraceTest(CoverageTest):
             b = 22
             ''',
             lines=[1, 2, 4, 5, 6, 7, 9, 10, 12, 13, 14, 16, 17, 18, 20, 21, 22, 23, 24],
-            missing="5-7, 13-14",
+            missing=missing,
         )
 
         assert self.last_module_name is not None
