@@ -25,8 +25,9 @@ from coverage import env
 from coverage.data import line_counts
 from coverage.files import abs_file, python_reported_file
 
+from tests import testenv
 from tests.coveragetest import CoverageTest, TESTS_DIR
-from tests.helpers import re_lines, re_lines_text
+from tests.helpers import re_line, re_lines, re_lines_text
 
 
 class ProcessTest(CoverageTest):
@@ -534,10 +535,12 @@ class ProcessTest(CoverageTest):
         assert py_out == "None\n"
 
         cov_out = self.run_command("coverage run showtrace.py")
-        if os.getenv('COVERAGE_TEST_TRACER', 'c') == 'c':
+        if testenv.C_TRACER:
             # If the C trace function is being tested, then regular running should have
             # the C function, which registers itself as f_trace.
             assert cov_out == "CTracer\n"
+        elif testenv.SYS_MON:
+            assert cov_out == "None\n"
         else:
             # If the Python trace function is being tested, then regular running will
             # also show the Python function.
@@ -1078,6 +1081,57 @@ class FailUnderTest(CoverageTest):
         assert expected == self.last_line_squeezed(out)
 
 
+class CoverageCoreTest(CoverageTest):
+    """Test that cores are chosen correctly."""
+    # This doesn't test failure modes, only successful requests.
+    try:
+        from coverage.tracer import CTracer
+        has_ctracer = True
+    except ImportError:
+        has_ctracer = False
+
+    def test_core_default(self) -> None:
+        self.del_environ("COVERAGE_TEST_CORES")
+        self.del_environ("COVERAGE_CORE")
+        self.make_file("numbers.py", "print(123, 456)")
+        out = self.run_command("coverage run --debug=sys numbers.py")
+        assert out.endswith("123 456\n")
+        core = re_line(r" core:", out).strip()
+        if self.has_ctracer:
+            assert core == "core: CTracer"
+        else:
+            assert core == "core: PyTracer"
+
+    @pytest.mark.skipif(not has_ctracer, reason="No CTracer to request")
+    def test_core_request_ctrace(self) -> None:
+        self.del_environ("COVERAGE_TEST_CORES")
+        self.set_environ("COVERAGE_CORE", "ctrace")
+        self.make_file("numbers.py", "print(123, 456)")
+        out = self.run_command("coverage run --debug=sys numbers.py")
+        assert out.endswith("123 456\n")
+        core = re_line(r" core:", out).strip()
+        assert core == "core: CTracer"
+
+    def test_core_request_pytrace(self) -> None:
+        self.del_environ("COVERAGE_TEST_CORES")
+        self.set_environ("COVERAGE_CORE", "pytrace")
+        self.make_file("numbers.py", "print(123, 456)")
+        out = self.run_command("coverage run --debug=sys numbers.py")
+        assert out.endswith("123 456\n")
+        core = re_line(r" core:", out).strip()
+        assert core == "core: PyTracer"
+
+    @pytest.mark.skipif(not env.PYBEHAVIOR.pep669, reason="No sys.monitoring to request")
+    def test_core_request_sysmon(self) -> None:
+        self.del_environ("COVERAGE_TEST_CORES")
+        self.set_environ("COVERAGE_CORE", "sysmon")
+        self.make_file("numbers.py", "print(123, 456)")
+        out = self.run_command("coverage run --debug=sys numbers.py")
+        assert out.endswith("123 456\n")
+        core = re_line(r" core:", out).strip()
+        assert core == "core: SysMonitor"
+
+
 class FailUnderNoFilesTest(CoverageTest):
     """Test that nothing to report results in an error exit status."""
     def test_report(self) -> None:
